# -*- coding: utf-8 -*-
"""
Texterra Python Interface
=========================

This package provides a Python interface for Texterra API.
Texterra is a toolkit for natural language processing and knowledge base utilization,
developed by a team of researchers at the Ivannikov Institute for System Programming.
Currently, English and Russian languages are supported.

See https://texterra.ispras.ru for more information about Texterra project.
"""
from .api import API
from .syntaxtree import SyntaxTree

__version__ = '1.0.0'
