from . import (
    languagedetection,
    sentencedetection,
    tokenization,
    lemmatization,
    namedentities,
    postagging,
    disambiguation,
    keyconcepts,
    spellingcorrection,
    subjectivitydetection,
    polaritydetection,
    syntaxdetection
)
